@@ -13,10 +13,10 @@ import (
 	"sync/atomic"
 	"time"
 
-	"github.com/canonical/go-dqlite"
-	"github.com/canonical/go-dqlite/client"
-	"github.com/canonical/go-dqlite/driver"
-	"github.com/canonical/go-dqlite/internal/protocol"
+	"github.com/canonical/go-dqlite/v2"
+	"github.com/canonical/go-dqlite/v2/client"
+	"github.com/canonical/go-dqlite/v2/driver"
+	"github.com/canonical/go-dqlite/v2/internal/protocol"
 	"github.com/pkg/errors"
 	"golang.org/x/sync/semaphore"
 )
@@ -40,6 +40,7 @@ type App struct {
 	tls             *tlsSetup
 	dialFunc        client.DialFunc
 	store           client.NodeStore
+	lc              *client.Connector
 	driver          *driver.Driver
 	driverName      string
 	log             client.LogFunc
@@ -239,10 +240,12 @@ func New(dir string, options ...Option) (app *App, err error) {
 		nodeBindAddress = nodeBindAddress[1:]
 	}
 
-	if runtime.GOOS != "linux" && nodeBindAddress[0] == '@' {
-		// Do not use abstract socket on other platforms and left trim "@"
-		nodeBindAddress = nodeBindAddress[1:]
-	}
+	lc := client.NewLeaderConnector(
+		store,
+		client.WithDialFunc(driverDial),
+		client.WithLogFunc(o.Log),
+		client.WithConcurrentLeaderConns(*o.ConcurrentLeaderConns),
+	)
 
 	app = &App{
 		id:              info.ID,
@@ -252,14 +255,15 @@ func New(dir string, options ...Option) (app *App, err error) {
 		nodeBindAddress: nodeBindAddress,
 		store:           store,
 		dialFunc:        driverDial,
+		lc:              lc,
 		driver:          driver,
 		driverName:      driverName,
 		log:             o.Log,
 		tls:             o.TLS,
 		ctx:             ctx,
 		stop:            stop,
-		runCh:           make(chan struct{}, 0),
-		readyCh:         make(chan struct{}, 0),
+		runCh:           make(chan struct{}),
+		readyCh:         make(chan struct{}),
 		voters:          o.Voters,
 		standbys:        o.StandBys,
 		roles:           RolesConfig{Voters: o.Voters, StandBys: o.StandBys},
@@ -272,7 +276,7 @@ func New(dir string, options ...Option) (app *App, err error) {
 		if err != nil {
 			return nil, fmt.Errorf("listen to %s: %w", info.Address, err)
 		}
-		proxyCh := make(chan struct{}, 0)
+		proxyCh := make(chan struct{})
 
 		app.listener = listener
 		app.proxyCh = proxyCh
@@ -330,7 +334,7 @@ func (a *App) Handover(ctx context.Context) error {
 	ctx, cancel = context.WithTimeout(ctx, time.Minute)
 	defer cancel()
 
-	cli, err := a.Leader(ctx)
+	cli, err := a.FindLeader(ctx)
 	if err != nil {
 		return fmt.Errorf("find leader: %w", err)
 	}
@@ -384,7 +388,7 @@ func (a *App) Handover(ctx context.Context) error {
 					return fmt.Errorf("transfer leadership: %w", err)
 				}
 			}
-			cli, err = a.Leader(ctx)
+			cli, err = a.FindLeader(ctx)
 			if err != nil {
 				return fmt.Errorf("find new leader: %w", err)
 			}
@@ -490,7 +494,9 @@ func (a *App) Open(ctx context.Context, database string) (*sql.DB, error) {
 	return db, nil
 }
 
-// Leader returns a client connected to the current cluster leader, if any.
+// Leader returns a client connected to the cluster leader.
+//
+// Prefer to use FindLeader instead unless you need to pass custom options.
 func (a *App) Leader(ctx context.Context, options ...client.Option) (*client.Client, error) {
 	allOptions := a.clientOptions()
 	allOptions = append(allOptions, options...)
@@ -498,6 +504,14 @@ func (a *App) Leader(ctx context.Context, options ...client.Option) (*client.Cli
 	return client.FindLeader(ctx, a.store, allOptions...)
 }
 
+// FindLeader returns a client connected to the cluster leader.
+//
+// Compared to Leader, this method avoids opening extra connections int many
+// cases, but doesn't accept custom options.
+func (a *App) FindLeader(ctx context.Context) (*client.Client, error) {
+	return a.lc.Connect(ctx)
+}
+
 // Client returns a client connected to the local node.
 func (a *App) Client(ctx context.Context) (*client.Client, error) {
 	return client.New(ctx, a.nodeBindAddress)
@@ -550,7 +564,7 @@ func (a *App) run(ctx context.Context, options *options, join bool) {
 			}
 			return
 		case <-time.After(delay):
-			cli, err := a.Leader(ctx)
+			cli, err := a.FindLeader(ctx)
 			if err != nil {
 				continue
 			}
@@ -744,13 +758,18 @@ func (a *App) makeRolesChanges(nodes []client.NodeInfo) RolesChanges {
 
 // Return the options to use for client.FindLeader() or client.New()
 func (a *App) clientOptions() []client.Option {
-	return []client.Option{client.WithDialFunc(a.dialFunc), client.WithLogFunc(a.log), client.WithConcurrentLeaderConns(*a.options.ConcurrentLeaderConns)}
+	return []client.Option{
+		client.WithDialFunc(a.dialFunc),
+		client.WithLogFunc(a.log),
+		client.WithConcurrentLeaderConns(*a.options.ConcurrentLeaderConns),
+	}
 }
 
 func (a *App) debug(format string, args ...interface{}) {
 	a.log(client.LogDebug, format, args...)
 }
 
+//lint:ignore U1000 not currently used but preserved for consistency
 func (a *App) info(format string, args ...interface{}) {
 	a.log(client.LogInfo, format, args...)
 }
