@@ -18,9 +18,9 @@ import (
 	"testing"
 	"time"
 
-	"github.com/canonical/go-dqlite"
-	"github.com/canonical/go-dqlite/app"
-	"github.com/canonical/go-dqlite/client"
+	"github.com/canonical/go-dqlite/v2"
+	"github.com/canonical/go-dqlite/v2/app"
+	"github.com/canonical/go-dqlite/v2/client"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 )
@@ -361,12 +361,12 @@ func TestHandover_VoterHonorFailureDomain(t *testing.T) {
 	require.NoError(t, err)
 	defer cli.Close()
 
-	cluster, err := cli.Cluster(context.Background())
+	_, err = cli.Cluster(context.Background())
 	require.NoError(t, err)
 
 	require.NoError(t, apps[2].Handover(context.Background()))
 
-	cluster, err = cli.Cluster(context.Background())
+	cluster, err := cli.Cluster(context.Background())
 	require.NoError(t, err)
 
 	assert.Equal(t, client.Voter, cluster[0].Role)
@@ -401,8 +401,13 @@ func TestHandover_GracefulShutdown(t *testing.T) {
 		defer cleanup()
 
 		addr := fmt.Sprintf("127.0.0.1:900%d", i+1)
+		log := func(l client.LogLevel, format string, a ...interface{}) {
+			format = fmt.Sprintf("%s - %d: %s: %s", time.Now().Format("15:04:01.000"), i, l.String(), format)
+			t.Logf(format, a...)
+		}
 		options := []app.Option{
 			app.WithAddress(addr),
+			app.WithLogFunc(log),
 		}
 		if i > 0 {
 			options = append(options, app.WithCluster([]string{"127.0.0.1:9001"}))
@@ -1292,8 +1297,8 @@ func Test_TxRowsAffected(t *testing.T) {
 CREATE TABLE test (
 	id            TEXT PRIMARY KEY,
 	value         INT
-);`);
-	require.NoError(t, err);
+);`)
+	require.NoError(t, err)
 
 	// Insert watermark
 	err = tx(context.Background(), db, func(ctx context.Context, tx *sql.Tx) error {
