@@ -9,8 +9,8 @@ import (
 	"path"
 	"time"
 
-	"github.com/canonical/go-dqlite/app"
-	"github.com/canonical/go-dqlite/client"
+	"github.com/canonical/go-dqlite/v2/app"
+	"github.com/canonical/go-dqlite/v2/client"
 )
 
 const (
@@ -85,7 +85,7 @@ func (bm *Benchmark) reportFiles() map[string]string {
 		reports := worker.report()
 		for w, report := range reports {
 			file := reportName(i, w)
-			allReports[file] = fmt.Sprintf("%s", report)
+			allReports[file] = report.String()
 		}
 	}
 	return allReports
