@@ -8,9 +8,8 @@ import (
 	"testing"
 	"time"
 
-	dqlite "github.com/canonical/go-dqlite"
-	"github.com/canonical/go-dqlite/client"
-	"github.com/canonical/go-dqlite/internal/protocol"
+	dqlite "github.com/canonical/go-dqlite/v2"
+	"github.com/canonical/go-dqlite/v2/client"
 	"github.com/stretchr/testify/assert"
 	"github.com/stretchr/testify/require"
 )
@@ -33,49 +32,6 @@ func TestClient_Leader(t *testing.T) {
 	assert.Equal(t, leader.Address, "@1001")
 }
 
-func TestClient_Dump(t *testing.T) {
-	node, cleanup := newNode(t)
-	defer cleanup()
-
-	ctx, cancel := context.WithTimeout(context.Background(), time.Second)
-	defer cancel()
-
-	client, err := client.New(ctx, node.BindAddress())
-	require.NoError(t, err)
-	defer client.Close()
-
-	// Open a database and create a test table.
-	request := protocol.Message{}
-	request.Init(4096)
-
-	response := protocol.Message{}
-	response.Init(4096)
-
-	protocol.EncodeOpen(&request, "test.db", 0, "volatile")
-
-	p := client.Protocol()
-	err = p.Call(ctx, &request, &response)
-	require.NoError(t, err)
-
-	db, err := protocol.DecodeDb(&response)
-	require.NoError(t, err)
-
-	protocol.EncodeExecSQLV0(&request, uint64(db), "CREATE TABLE foo (n INT)", nil)
-
-	err = p.Call(ctx, &request, &response)
-	require.NoError(t, err)
-
-	files, err := client.Dump(ctx, "test.db")
-	require.NoError(t, err)
-
-	require.Len(t, files, 2)
-	assert.Equal(t, "test.db", files[0].Name)
-	assert.Equal(t, 4096, len(files[0].Data))
-
-	assert.Equal(t, "test.db-wal", files[1].Name)
-	assert.Equal(t, 8272, len(files[1].Data))
-}
-
 func TestClient_Cluster(t *testing.T) {
 	node, cleanup := newNode(t)
 	defer cleanup()
