@@ -8,8 +8,8 @@ import (
 	"path/filepath"
 	"time"
 
-	"github.com/canonical/go-dqlite/app"
-	"github.com/canonical/go-dqlite/benchmark"
+	"github.com/canonical/go-dqlite/v2/app"
+	"github.com/canonical/go-dqlite/v2/benchmark"
 	"github.com/pkg/errors"
 	"github.com/spf13/cobra"
 	"golang.org/x/sys/unix"
@@ -90,10 +90,8 @@ func main() {
 			ch := signalChannel()
 			if !driver {
 				fmt.Println("Benchmark client ready. Send signal to abort or when done.")
-				select {
-				case <-ch:
-					return nil
-				}
+				<-ch
+				return nil
 			}
 
 			if len(*cluster) == 0 {
