@@ -11,9 +11,7 @@ import (
 	"time"
 
 	"github.com/Rican7/retry"
-	"github.com/Rican7/retry/backoff"
-	"github.com/Rican7/retry/strategy"
-	"github.com/canonical/go-dqlite/logging"
+	"github.com/canonical/go-dqlite/v2/logging"
 	"github.com/pkg/errors"
 	"golang.org/x/sync/semaphore"
 )
@@ -24,91 +22,185 @@ const MaxConcurrentLeaderConns int64 = 10
 // DialFunc is a function that can be used to establish a network connection.
 type DialFunc func(context.Context, string) (net.Conn, error)
 
-// Connector is in charge of creating a dqlite SQL client connected to the
-// current leader of a cluster.
-type Connector struct {
-	id     uint64       // Conn ID to use when registering against the server.
-	store  NodeStore    // Used to get and update current cluster servers.
-	config Config       // Connection parameters.
-	log    logging.Func // Logging function.
+// LeaderTracker remembers the address of the cluster leader, and possibly
+// holds a reusable connection to it.
+type LeaderTracker struct {
+	mu                  sync.RWMutex
+	lastKnownLeaderAddr string
+
+	proto *Protocol
 }
 
-// NewConnector returns a new connector that can be used by a dqlite driver to
-// create new clients connected to a leader dqlite server.
-func NewConnector(id uint64, store NodeStore, config Config, log logging.Func) *Connector {
+func (lt *LeaderTracker) GetLeaderAddr() string {
+	lt.mu.RLock()
+	defer lt.mu.RUnlock()
+
+	return lt.lastKnownLeaderAddr
+}
+
+func (lt *LeaderTracker) SetLeaderAddr(address string) {
+	lt.mu.Lock()
+	defer lt.mu.Unlock()
+
+	lt.lastKnownLeaderAddr = address
+}
+
+func (lt *LeaderTracker) UnsetLeaderAddr() {
+	lt.mu.Lock()
+	defer lt.mu.Unlock()
+
+	lt.lastKnownLeaderAddr = ""
+}
+
+func (lt *LeaderTracker) TakeSharedProtocol() (proto *Protocol) {
+	lt.mu.Lock()
+	defer lt.mu.Unlock()
+
+	if proto, lt.proto = lt.proto, nil; proto != nil {
+		proto.lt = lt
+	}
+	return
+}
+
+func (lt *LeaderTracker) DonateSharedProtocol(proto *Protocol) (accepted bool) {
+	lt.mu.Lock()
+	defer lt.mu.Unlock()
+
+	if accepted = lt.proto == nil; accepted {
+		lt.proto = proto
+	}
+	return
+}
+
+type Connector struct {
+	clientID    uint64 // Conn ID to use when registering against the server.
+	store       NodeStore
+	nodeID      uint64
+	nodeAddress string
+	lt          *LeaderTracker
+	config      Config       // Connection parameters.
+	log         logging.Func // Logging function.
+}
+
+// NewConnector returns a Connector that will connect to the current cluster
+// leader.
+func NewLeaderConnector(store NodeStore, config Config, log logging.Func) *Connector {
 	if config.Dial == nil {
 		config.Dial = Dial
 	}
-
 	if config.DialTimeout == 0 {
 		config.DialTimeout = 5 * time.Second
 	}
-
 	if config.AttemptTimeout == 0 {
 		config.AttemptTimeout = 15 * time.Second
 	}
-
 	if config.BackoffFactor == 0 {
 		config.BackoffFactor = 100 * time.Millisecond
 	}
-
 	if config.BackoffCap == 0 {
 		config.BackoffCap = time.Second
 	}
-
 	if config.ConcurrentLeaderConns == 0 {
 		config.ConcurrentLeaderConns = MaxConcurrentLeaderConns
 	}
 
-	connector := &Connector{
-		id:     id,
+	return &Connector{
 		store:  store,
+		lt:     &LeaderTracker{},
 		config: config,
 		log:    log,
 	}
-
-	return connector
 }
 
-// Connect finds the leader server and returns a connection to it.
-//
-// If the connector is stopped before a leader is found, nil is returned.
+// NewDirectConnector returns a Connector that will connect to the node with
+// the given ID and address.
+func NewDirectConnector(id uint64, address string, config Config, log logging.Func) *Connector {
+	if config.Dial == nil {
+		config.Dial = Dial
+	}
+	if config.DialTimeout == 0 {
+		config.DialTimeout = 5 * time.Second
+	}
+	if config.AttemptTimeout == 0 {
+		config.AttemptTimeout = 15 * time.Second
+	}
+	if config.BackoffFactor == 0 {
+		config.BackoffFactor = 100 * time.Millisecond
+	}
+	if config.BackoffCap == 0 {
+		config.BackoffCap = time.Second
+	}
+	if config.ConcurrentLeaderConns == 0 {
+		config.ConcurrentLeaderConns = MaxConcurrentLeaderConns
+	}
+
+	return &Connector{
+		nodeID:      id,
+		nodeAddress: address,
+		lt:          &LeaderTracker{},
+		config:      config,
+		log:         log,
+	}
+}
+
+// Connect opens a new Protocol based on the Connector's configuration.
 func (c *Connector) Connect(ctx context.Context) (*Protocol, error) {
+	if c.nodeID != 0 {
+		ctx, cancel := context.WithTimeout(ctx, c.config.AttemptTimeout)
+		defer cancel()
+		conn, err := c.config.Dial(ctx, c.nodeAddress)
+		if err != nil {
+			return nil, errors.Wrap(err, "dial")
+		}
+		version := VersionOne
+		protocol, err := Handshake(ctx, conn, version, c.nodeAddress)
+		if err == errBadProtocol {
+			c.log(logging.Warn, "unsupported protocol %d, attempt with legacy", version)
+			version = VersionLegacy
+			protocol, err = Handshake(ctx, conn, version, c.nodeAddress)
+		}
+		if err != nil {
+			conn.Close()
+			return nil, errors.Wrap(err, "handshake")
+		}
+		return protocol, nil
+	}
+
+	if c.config.PermitShared {
+		if sharedProto := c.lt.TakeSharedProtocol(); sharedProto != nil {
+			if leaderAddr, err := askLeader(ctx, sharedProto); err == nil && sharedProto.addr == leaderAddr {
+				c.log(logging.Debug, "reusing shared connection to %s", sharedProto.addr)
+				c.lt.SetLeaderAddr(leaderAddr)
+				return sharedProto, nil
+			}
+			c.log(logging.Debug, "discarding shared connection to %s", sharedProto.addr)
+			sharedProto.Bad()
+			sharedProto.Close()
+		}
+	}
+
 	var protocol *Protocol
-
-	strategies := makeRetryStrategies(c.config.BackoffFactor, c.config.BackoffCap, c.config.RetryLimit)
-
-	// The retry strategy should be configured to retry indefinitely, until
-	// the given context is done.
 	err := retry.Retry(func(attempt uint) error {
 		log := func(l logging.Level, format string, a ...interface{}) {
 			format = fmt.Sprintf("attempt %d: ", attempt) + format
 			c.log(l, format, a...)
 		}
 
-		select {
-		case <-ctx.Done():
-			// Stop retrying
-			return nil
-		default:
+		if attempt > 1 {
+			select {
+			case <-ctx.Done():
+				// Stop retrying
+				return nil
+			default:
+			}
 		}
 
 		var err error
 		protocol, err = c.connectAttemptAll(ctx, log)
-		if err != nil {
-			return err
-		}
+		return err
+	}, c.config.RetryStrategies()...)
 
-		return nil
-	}, strategies...)
-
-	if err != nil {
-		// We exhausted the number of retries allowed by the configured
-		// strategy.
-		return nil, ErrNoAvailableLeader
-	}
-
-	if ctx.Err() != nil {
+	if err != nil || ctx.Err() != nil {
 		return nil, ErrNoAvailableLeader
 	}
 
@@ -119,18 +211,38 @@ func (c *Connector) Connect(ctx context.Context) (*Protocol, error) {
 		panic("no protocol object")
 	}
 
+	c.lt.SetLeaderAddr(protocol.addr)
+	if c.config.PermitShared {
+		protocol.lt = c.lt
+	}
 	return protocol, nil
 }
 
-// Make a single attempt to establish a connection to the leader server trying
-// all addresses available in the store.
+// connectAttemptAll tries to establish a new connection to the cluster leader.
+//
+// First, if the address of the last known leader has been recorded, try
+// to connect to that server and confirm its leadership. This is a fast path
+// for stable clusters that avoids opening lots of connections. If that fails,
+// fall back to probing all servers in parallel, checking whether each
+// is the leader itself or knows who the leader is.
 func (c *Connector) connectAttemptAll(ctx context.Context, log logging.Func) (*Protocol, error) {
+	if addr := c.lt.GetLeaderAddr(); addr != "" {
+		// TODO In the event of failure, we could still use the second
+		// return value to guide the next stage of the search.
+		if proto, _, _ := c.connectAttemptOne(ctx, ctx, addr, log); proto != nil {
+			log(logging.Debug, "server %s: connected on fast path", addr)
+			return proto, nil
+		}
+		c.lt.UnsetLeaderAddr()
+	}
+
 	servers, err := c.store.Get(ctx)
 	if err != nil {
 		return nil, errors.Wrap(err, "get servers")
 	}
-
-	// Sort servers by Role, from low to high.
+	// Probe voters before standbys before spares. Only voters can potentially
+	// be the leader, and standbys are more likely to know who the leader is
+	// than spares since they participate more in the cluster.
 	sort.Slice(servers, func(i, j int) bool {
 		return servers[i].Role < servers[j].Role
 	})
@@ -144,102 +256,64 @@ func (c *Connector) connectAttemptAll(ctx context.Context, log logging.Func) (*P
 	ctx, cancel := context.WithCancel(ctx)
 	defer cancel()
 
+	leaderCh := make(chan *Protocol)
 	sem := semaphore.NewWeighted(c.config.ConcurrentLeaderConns)
-
-	protocolCh := make(chan *Protocol)
-
 	wg := &sync.WaitGroup{}
 	wg.Add(len(servers))
-
 	go func() {
 		wg.Wait()
-		close(protocolCh)
+		close(leaderCh)
 	}()
-
-	// Make an attempt for each address until we find the leader.
 	for _, server := range servers {
-		go func(server NodeInfo, pc chan<- *Protocol) {
+		go func(server NodeInfo) {
 			defer wg.Done()
 
 			if err := sem.Acquire(ctx, 1); err != nil {
+				log(logging.Warn, "server %s: %v", server.Address, err)
 				return
 			}
 			defer sem.Release(1)
 
-			if ctx.Err() != nil {
-				return
-			}
-
-			log := func(l logging.Level, format string, a ...interface{}) {
-				format = fmt.Sprintf("server %s: ", server.Address) + format
-				log(l, format, a...)
-			}
-
-			ctx, cancel := context.WithTimeout(ctx, c.config.AttemptTimeout)
-			defer cancel()
-
 			protocol, leader, err := c.connectAttemptOne(origCtx, ctx, server.Address, log)
 			if err != nil {
-				// This server is unavailable, try with the next target.
-				log(logging.Warn, err.Error())
+				log(logging.Warn, "server %s: %v", server.Address, err)
 				return
-			}
-			if protocol != nil {
-				// We found the leader
-				log(logging.Debug, "connected")
-				pc <- protocol
+			} else if protocol != nil {
+				leaderCh <- protocol
 				return
-			}
-			if leader == "" {
-				// This server does not know who the current leader is,
-				// try with the next target.
-				log(logging.Warn, "no known leader")
+			} else if leader == "" {
+				log(logging.Warn, "server %s: no known leader", server.Address)
 				return
 			}
 
-			// If we get here, it means this server reported that another
-			// server is the leader, let's close the connection to this
-			// server and try with the suggested one.
-			log(logging.Debug, "connect to reported leader %s", leader)
-
-			ctx, cancel = context.WithTimeout(ctx, c.config.AttemptTimeout)
-			defer cancel()
-
+			// Try the server that the original server thinks is the leader.
+			log(logging.Debug, "server %s: connect to reported leader %s", server.Address, leader)
 			protocol, _, err = c.connectAttemptOne(origCtx, ctx, leader, log)
 			if err != nil {
-				// The leader reported by the previous server is
-				// unavailable, try with the next target.
-				log(logging.Warn, "reported leader unavailable err=%v", err)
+				log(logging.Warn, "server %s: %v", leader, err)
+				return
+			} else if protocol == nil {
+				log(logging.Warn, "server %s: reported leader server is not the leader", leader)
 				return
 			}
-			if protocol == nil {
-				// The leader reported by the target server does not consider itself
-				// the leader, try with the next target.
-				log(logging.Warn, "reported leader server is not the leader")
-				return
-			}
-			log(logging.Debug, "connected")
-			pc <- protocol
-		}(server, protocolCh)
+			leaderCh <- protocol
+		}(server)
 	}
 
-	// Read from protocol chan, cancel context
-	protocol, ok := <-protocolCh
+	leader, ok := <-leaderCh
+	cancel()
 	if !ok {
 		return nil, ErrNoAvailableLeader
 	}
-
-	cancel()
-
-	for extra := range protocolCh {
+	log(logging.Debug, "server %s: connected on fallback path", leader.addr)
+	for extra := range leaderCh {
 		extra.Close()
 	}
-
-	return protocol, nil
+	return leader, nil
 }
 
 // Perform the initial handshake using the given protocol version.
-func Handshake(ctx context.Context, conn net.Conn, version uint64) (*Protocol, error) {
+func Handshake(ctx context.Context, conn net.Conn, version uint64, addr string) (*Protocol, error) {
 	// Latest protocol version.
 	protocol := make([]byte, 8)
 	binary.LittleEndian.PutUint64(protocol, version)
@@ -259,7 +333,7 @@ func Handshake(ctx context.Context, conn net.Conn, version uint64) (*Protocol, e
 		return nil, errors.Wrap(io.ErrShortWrite, "short handshake write")
 	}
 
-	return newProtocol(version, conn), nil
+	return &Protocol{conn: conn, version: version, addr: addr}, nil
 }
 
 // Connect to the given dqlite server and check if it's the leader.
@@ -276,9 +350,21 @@ func (c *Connector) connectAttemptOne(
 	dialCtx context.Context,
 	ctx context.Context,
 	address string,
-	log logging.Func,
+	origLog logging.Func,
 ) (*Protocol, string, error) {
-	dialCtx, cancel := context.WithTimeout(dialCtx, c.config.DialTimeout)
+	log := func(l logging.Level, format string, a ...interface{}) {
+		format = fmt.Sprintf("server %s: ", address) + format
+		origLog(l, format, a...)
+	}
+
+	if ctx.Err() != nil {
+		return nil, "", ctx.Err()
+	}
+
+	ctx, cancel := context.WithTimeout(ctx, c.config.AttemptTimeout)
+	defer cancel()
+
+	dialCtx, cancel = context.WithTimeout(dialCtx, c.config.DialTimeout)
 	defer cancel()
 
 	// Establish the connection.
@@ -288,43 +374,22 @@ func (c *Connector) connectAttemptOne(
 	}
 
 	version := VersionOne
-	protocol, err := Handshake(ctx, conn, version)
+	protocol, err := Handshake(ctx, conn, version, address)
 	if err == errBadProtocol {
 		log(logging.Warn, "unsupported protocol %d, attempt with legacy", version)
 		version = VersionLegacy
-		protocol, err = Handshake(ctx, conn, version)
+		protocol, err = Handshake(ctx, conn, version, address)
 	}
 	if err != nil {
 		conn.Close()
 		return nil, "", err
 	}
 
-	// Send the initial Leader request.
-	request := Message{}
-	request.Init(16)
-	response := Message{}
-	response.Init(512)
-
-	EncodeLeader(&request)
-
-	if err := protocol.Call(ctx, &request, &response); err != nil {
-		protocol.Close()
-		cause := errors.Cause(err)
-		// Best-effort detection of a pre-1.0 dqlite node: when sent
-		// version 1 it should close the connection immediately.
-		if err, ok := cause.(*net.OpError); ok && !err.Timeout() || cause == io.EOF {
-			return nil, "", errBadProtocol
-		}
-
-		return nil, "", err
-	}
-
-	_, leader, err := DecodeNodeCompat(protocol, &response)
+	leader, err := askLeader(ctx, protocol)
 	if err != nil {
 		protocol.Close()
 		return nil, "", err
 	}
-
 	switch leader {
 	case "":
 		// Currently this server does not know about any leader.
@@ -332,10 +397,12 @@ func (c *Connector) connectAttemptOne(
 		return nil, "", nil
 	case address:
 		// This server is the leader, register ourselves and return.
-		request.reset()
-		response.reset()
+		request := Message{}
+		request.Init(16)
+		response := Message{}
+		response.Init(512)
 
-		EncodeClient(&request, c.id)
+		EncodeClient(&request, c.clientID)
 
 		if err := protocol.Call(ctx, &request, &response); err != nil {
 			protocol.Close()
@@ -360,34 +427,32 @@ func (c *Connector) connectAttemptOne(
 	}
 }
 
-// Return a retry strategy with exponential backoff, capped at the given amount
-// of time and possibly with a maximum number of retries.
-func makeRetryStrategies(factor, cap time.Duration, limit uint) []strategy.Strategy {
-	limit += 1 // Fix for change in behavior: https://github.com/Rican7/retry/pull/12
-	backoff := backoff.BinaryExponential(factor)
+// TODO move client logic including Leader method to Protocol,
+// and get rid of this.
+func askLeader(ctx context.Context, protocol *Protocol) (string, error) {
+	request := Message{}
+	request.Init(16)
+	response := Message{}
+	response.Init(512)
 
-	strategies := []strategy.Strategy{}
+	EncodeLeader(&request)
 
-	if limit > 1 {
-		strategies = append(strategies, strategy.Limit(limit))
+	if err := protocol.Call(ctx, &request, &response); err != nil {
+		cause := errors.Cause(err)
+		// Best-effort detection of a pre-1.0 dqlite node: when sent
+		// version 1 it should close the connection immediately.
+		if err, ok := cause.(*net.OpError); ok && !err.Timeout() || cause == io.EOF {
+			return "", errBadProtocol
+		}
+
+		return "", err
 	}
 
-	strategies = append(strategies,
-		func(attempt uint) bool {
-			if attempt > 0 {
-				duration := backoff(attempt)
-				// Duration might be negative in case of integer overflow.
-				if duration > cap || duration <= 0 {
-					duration = cap
-				}
-				time.Sleep(duration)
-			}
-
-			return true
-		},
-	)
-
-	return strategies
+	_, leader, err := DecodeNodeCompat(protocol, &response)
+	if err != nil {
+		return "", err
+	}
+	return leader, nil
 }
 
 var errBadProtocol = fmt.Errorf("bad protocol")
