@@ -7,11 +7,7 @@ import (
 // Client errors.
 var (
 	ErrNoAvailableLeader = fmt.Errorf("no available dqlite leader server found")
-	errStop              = fmt.Errorf("connector was stopped")
-	errStaleLeader       = fmt.Errorf("server has lost leadership")
-	errNotClustered      = fmt.Errorf("server is not clustered")
 	errNegativeRead      = fmt.Errorf("reader returned negative count from Read")
-	errMessageEOF        = fmt.Errorf("message eof")
 )
 
 // ErrRequest is returned in case of request failure.
