@@ -11,10 +11,11 @@ import (
 	"path/filepath"
 	"strconv"
 	"strings"
+	"text/tabwriter"
 
-	"github.com/canonical/go-dqlite"
-	"github.com/canonical/go-dqlite/client"
-	"github.com/canonical/go-dqlite/driver"
+	"github.com/canonical/go-dqlite/v2"
+	"github.com/canonical/go-dqlite/v2/client"
+	"github.com/canonical/go-dqlite/v2/driver"
 )
 
 // Shell can be used to implement interactive prompts for inspecting a dqlite
@@ -130,7 +131,7 @@ func (s *Shell) processCluster(ctx context.Context, line string) (string, error)
 		}
 		var indented bytes.Buffer
 		json.Indent(&indented, data, "", "\t")
-		result = string(indented.Bytes())
+		result = indented.String()
 	}
 
 	return result, nil
@@ -204,7 +205,7 @@ func (s *Shell) processDescribe(ctx context.Context, line string) (string, error
 		}
 		var indented bytes.Buffer
 		json.Indent(&indented, data, "", "\t")
-		result = string(indented.Bytes())
+		result = indented.String()
 	}
 
 	return result, nil
@@ -249,6 +250,7 @@ func (s *Shell) processDump(ctx context.Context, line string) (string, error) {
 func (s *Shell) processReconfigure(ctx context.Context, line string) (string, error) {
 	parts := strings.Split(line, " ")
 	if len(parts) != 3 {
+		//lint:ignore ST1005 intentional long prosy error message
 		return "NOK", fmt.Errorf("bad command format, should be: .reconfigure <dir> <clusteryaml>\n" +
 			"Args:\n" +
 			"\tdir - Directory of node with up to date data\n" +
@@ -340,6 +342,12 @@ func (s *Shell) processQuery(ctx context.Context, line string) (string, error) {
 	n := len(columns)
 
 	var sb strings.Builder
+	writer := tabwriter.NewWriter(&sb, 0, 8, 1, '\t', 0)
+	for _, col := range columns {
+		fmt.Fprintf(writer, "%s\t", col)
+	}
+	fmt.Fprintln(writer)
+
 	for rows.Next() {
 		row := make([]interface{}, n)
 		rowPointers := make([]interface{}, n)
@@ -355,14 +363,10 @@ func (s *Shell) processQuery(ctx context.Context, line string) (string, error) {
 			return "", err
 		}
 
-		for i, column := range row {
-			if i == 0 {
-				fmt.Fprintf(&sb, "%v", column)
-			} else {
-				fmt.Fprintf(&sb, "|%v", column)
-			}
+		for _, column := range row {
+			fmt.Fprintf(writer, "%v\t", column)
 		}
-		sb.WriteByte('\n')
+		fmt.Fprintln(writer)
 	}
 
 	if err := rows.Err(); err != nil {
@@ -377,26 +381,9 @@ func (s *Shell) processQuery(ctx context.Context, line string) (string, error) {
 		return "", fmt.Errorf("commit: %w", err)
 	}
 
+	if err := writer.Flush(); err != nil {
+		return "", fmt.Errorf("flush: %w", err)
+	}
+
 	return strings.TrimRight(sb.String(), "\n"), nil
 }
-
-func (s *Shell) processExec(ctx context.Context, line string) error {
-	tx, err := s.db.BeginTx(ctx, nil)
-	if err != nil {
-		return err
-	}
-
-	if _, err := tx.Exec(line); err != nil {
-		err = fmt.Errorf("exec: %w", err)
-		if rbErr := tx.Rollback(); rbErr != nil {
-			return fmt.Errorf("unable to rollback: %v", err)
-		}
-		return err
-	}
-
-	if err := tx.Commit(); err != nil {
-		return fmt.Errorf("commit: %w", err)
-	}
-
-	return nil
-}
